%include %{_rpmconfigdir}/macros.python2

# Don't process requires for .py* files, since it massively increases build time
# and we already know that we depend Python -- we're building it!
%define __requires_exclude_from ^%{_libdir}/.*[.]py[oc]?$

%define major 2
%define minor 7
%define fix 18
%define priority %(printf '%02d%02d' %{major} %{minor})

Name: python%{major}
Version: %{major}.%{minor}.%{fix}
Release: 7
License: Python-2.0
Summary: Python 2 Programming Language
Url: https://www.python.org

BuildRequires: openssl-1.1-devel
BuildRequires: zlib-devel
BuildRequires: expat-devel
BuildRequires: libffi-devel
BuildRequires: xz-devel
BuildRequires: bzip2-devel
BuildRequires: ncurses-devel
BuildRequires: readline-devel
BuildRequires: sqlite3-devel
BuildRequires: libutil-devel >= 0.7

# if this is available, _locale will link to it
# Is there a benefit?
# BuildRequires: libintl-devel

Requires: libutil2 >= 0.7
Requires(post): update-alternatives
Requires(postun): update-alternatives

Source0: https://www.python.org/ftp/python/%{version}/Python-%{version}.tar.xz
Source1: python2-sitecustomize.py

Patch0: python2-configure.patch
Patch1: python2-Lib_distutils_util.py.patch
Patch2: python2-Lib_sysconfig.py.patch
Patch3: python2-Modules__ctypes_libffi_configure.patch
Patch4: Python-2.7.12-aix.patch
Patch5: Python-2.7.12-aixsetup.patch
Patch6: Python-2.7.12-ldshared.patch
Patch7: Python-2.7.12-linkso.patch
Patch8: Python-2.7.12-pthread-stack-size.patch
Patch9: Python-2.7.12-termios.patch
Patch10: Python-2.7.12-test.patch
Patch11: Python-2.7.13-semaphore.patch
Patch12: python2-o_nofollow.patch
Patch13: python2-use-libutil-for-flock.patch

%description
Python is a programming language that lets you work more quickly and integrate your systems more effectively.

You can learn to use Python and see almost immediate gains in productivity and lower maintenance costs.


%package devel
Summary: Python 2 development files
Requires: %{name} = %{version}-%{release}

%description devel
Python is a programming language that lets you work more quickly and integrate your systems more effectively.

You can learn to use Python and see almost immediate gains in productivity and lower maintenance costs.

%prep
%setup -q -n Python-%{version}

%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1
%patch10 -p1
%patch11 -p1
%patch12 -p1
%patch13 -p1

%build

# NOTE: To build with IPV6 support you need the following PTFs applied:
# 7.3: MF63822
# 7.2: MF63833
# 7.1: MF63834
#
# Or you may hit this error during configure:
#
# Fatal: You must get working getaddrinfo() function.
#        or you can specify "--disable-ipv6".

# Currently SLIC is limited to supporting AT_FDCWD for dirfd on *at APIs :(
# Since Python internal code will use *at APIs when available, and expects them
# to support more than just AT_FDCWD, we have to disable them or you will just
# get lots of EINVAL returns.

%configure \
  CFLAGS="-std=c99 $CFLAGS -I/QOpenSys/pkgs/include/ncurses" \
  CPPFLAGS="-D_XOPEN_SOURCE=700 -D_ALL_SOURCE -I/QOpenSys/pkgs/include/ncurses" \
  LDFLAGS="-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" \
  ARFLAGS="-X${OBJECT_MODE} rc" \
  ac_cv_func_clock_settime=no \
  ac_cv_func_posix_fadvise=no \
  ac_cv_func_posix_fallocate=no \
  ac_cv_func_sched_rr_get_interval=no \
  ac_cv_func_sched_setparam=no \
  ac_cv_func_sched_setscheduler=no \
  ac_cv_func_faccessat=no \
  ac_cv_func_fchmodat=no \
  ac_cv_func_fchownat=no \
  ac_cv_func_fstatat=no \
  ac_cv_func_futimesat=no \
  ac_cv_func_linkat=no \
  ac_cv_func_lstat=yes \
  ac_cv_func_mkdirat=no \
  ac_cv_func_mkfifoat=no \
  ac_cv_func_mknodat=no \
  ac_cv_func_openat=no \
  ac_cv_func_readlinkat=no \
  ac_cv_func_renameat=no \
  ac_cv_func_symlinkat=no \
  ac_cv_func_unlinkat=no \
  ac_cv_func_utimensat=no \
  --with-gcc \
  --with-threads \
  --with-system-expat \
  --with-system-ffi \
  --with-ensurepip=no \
  --without-computed-gotos \
  --enable-ipv6 \
  --enable-shared
  #end

# TODO: Ensure -maix32/-maix64 is passed to gcc when used as the linker. This is hacky. Fix it properly
perl -p -i -e "s|ld_so_aix \$(CC)|ld_so_aix \$(CC) -maix${OBJECT_MODE}|" Makefile

perl -p -i -e 's|#! /usr/local/bin/python|#! %{_bindir}/python2|' Lib/cgi.py # Already patched

gmake

# Hack to re-build python binary without -brtl
mv python python.bak
gcc -pthread "-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" -Wl,-bE:Modules/python.exp -lld -o python \
		Modules/python.o \
		libpython2.7.so -ldl  -lutil   -lm \
        -Wl,-bnortl -Wl,-blibsuff:so
touch -r python.bak python

%install

gmake install DESTDIR=%{buildroot}

rm %{buildroot}/%{_bindir}/python
rm %{buildroot}/%{_bindir}/python-config
rm %{buildroot}/%{_bindir}/2to3
rm %{buildroot}/%{_bindir}/idle

%define site_packages "%{buildroot}/%{_libdir}/python%{major}.%{minor}/site-packages"

cp %{SOURCE1} %{site_packages}/sitecustomize.py

%post
%{_bindir}/update-alternatives --install %{_bindir}/python python %{_bindir}/python%{major}.%{minor} %{priority}

%postun
%{_bindir}/update-alternatives --remove python %{_bindir}/python

%files
%defattr(-, qsys, *none)

%{_bindir}/pydoc
%{_bindir}/python%{major}
%{_bindir}/python%{major}.%{minor}
%{_bindir}/smtpd.py

%ghost %{_bindir}/python

%{_libdir}/libpython%{major}.%{minor}.so

%{_libdir}/python%{major}.%{minor}

%{_libdir}/python%{major}.%{minor}/lib-dynload/_bisect.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_cn.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_hk.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_iso2022.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_jp.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_kr.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_tw.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_collections.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_csv.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ctypes.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ctypes_test.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_curses.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_curses_panel.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_elementtree.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_functools.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_hashlib.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_heapq.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_hotshot.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_io.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_json.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_locale.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_lsprof.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_multibytecodec.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_multiprocessing.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_random.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_socket.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sqlite3.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ssl.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_struct.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testcapi.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/array.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/audioop.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/binascii.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/bz2.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/cPickle.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/cStringIO.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/cmath.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/crypt.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/datetime.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/dbm.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/fcntl.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/future_builtins.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/grp.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/itertools.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/math.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/mmap.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/nis.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/operator.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/parser.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/pyexpat.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/readline.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/resource.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/select.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/strop.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/syslog.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/termios.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/time.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/unicodedata.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/xxsubtype.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/zlib.so



%{_mandir}/man1/*

%files devel
%defattr(-, qsys, *none)

%{_bindir}/python%{major}-config
%{_bindir}/python%{major}.%{minor}-config

%{_includedir}/python%{major}.%{minor}
%{_libdir}/pkgconfig/*

%changelog
* Thu Nov 02 2023 Kevin Adler <kadler@us.ibm.com>
- Build python2 binary with -bnortl to allow packages to load multiple OpenSSL
  without conflicts (eg. libcurl linked with OpenSSL 3)

* Thu Feb 09 2023 Kevin Adler <kadler@us.ibm.com>
- BuildRequire openssl-1.1-devel instead of openssl-devel

* Tue Mar 22 2022 Abdirahim Musse <abdirahim.musse@ibm.com> - 2.7.18
- Stop passing -lutil to python extensions

* Thu Nov 4 2021 Jason Hansen <stymie@us.ibm.com>
- Intra-package Requires need version and release

* Fri Jul 17 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Ghost python alternative

* Tue Jul 14 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Move python*-config to devel package

* Tue Jun 23 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Fix missing colon after Requires tag

* Fri Jun 19 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Use update-alternatives

* Mon Jun 08 2020 Kevin Adler <kadler@us.ibm.com>
- Change install requirement from libutil-devel to libutil2

* Tue May 12 2020 Naveen Ram <naveen.ram@ibm.com> - 2.7.18
- Fix errors building fcntl when using libutil 0.7
- Require libutil 0.7 for fix to work properly
- Include path to ncruses in CFLAGS

* Mon May 04 2020 Naveen Ram <naveen.ram@ibm.com> - 2.7.18
- Update to 2.7.18

* Mon Oct 21 2019 Xu Meng <mengxumx@cn.ibm.com> - 2.7.17
- Update to 2.7.17
- Fix CVE-2019-16935
- Fix CVE-2019-9948
- Fix CVE-2019-9947

* Wed Sep 25 2019 Kevin Adler <kadler@us.ibm.com>
- Disable broken O_NOFOLLOW support on IBM i 7.2

* Wed Aug 07 2019 Mark Irish <mirish@ibm.com> - 2.7.16
- Update openssl-dev version to >= 1.1.1

* Fri Mar 15 2019 - Kevin Adler <kadler@us.ibm.com>
- Require libutil 0.6 (libutil.so.2)

* Tue Mar 05 2019 - Kevin Adler <kadler@us.ibm.com>
- Update to 2.7.16

* Mon Jan 14 2019 - Kevin Adler <kadler@us.ibm.com>
- Speed up automatic dependency generation
- Fix invalid autoconf checks before the fact instead of after

* Wed Jan 09 2019 - Kevin Adler <kadler@us.ibm.com>
- Require fixed libutil

* Tue Jul 10 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Put replacement of /usr/local/bin/python back

* Tue Jul 10 2018 - mengxumx <mengxumx@cn.ibm.com>
- Python build fails if using make -jxx

* Sun Jul 08 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Added LINKCC=cc

* Fri Jun 29 2018 - mengxumx <mengxumx@cn.ibm.com>
- Upgrade to 2.7.15

* Thu Mar 22 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Update to 2.7.14

* Sat Mar 03 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Added patch for named semaphore problem

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Add devel package

* Fri Feb 16 2018 - Kevin Adler <kadler@us.ibm.com>
- Build _locale, needed by YUM

* Tue Feb 13 2018 - Kevin Adler <kadler@us.ibm.com>
- Build bzip2 support, which yum requires

* Tue Feb 13 2018 - Kevin Adler <kadler@us.ibm.com>
- Ensure python builds with OpenSSL 1.0.2

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Add missing sitecustomize.py

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Build with expat, libffi, and ncurses

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Add missing sed-gnu build requirement

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Remove unused sitecustomize.py

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Define _POSIX_SOURCE and _ALL_SOURCE to add various POSIX features to Python,
  fixing an issue with yum running in C locale

* Wed Dec 20 2017 - Kevin Adler <kadler@us.ibm.com>
- Add patches from AIX toolbox and ignore locale package

* Wed Nov 22 2017 - Kevin Adler <kadler@us.ibm.com>
- Add python2 package
